# 
# usage: 
#   add the following calls to the end of the .bashrc/.profile cfg files
#   .profile: (PATH and other env vars)
#      . $HOME/bin/cfg/loop.sh ; fct_loop profile $* ; unset fct_loop
#   .bashrc: (interactive stuffs)
#      . $HOME/bin/cfg/loop.sh ; fct_loop bashrc $* ; unset fct_loop
#
# if this script is sometimes run with dash (not bash!)
# (by gnome on ubuntu/debian for the .profile)
#  => avoid bashisms (e.g. ~ becomes $HOME)

function add2env()
{
    #verb=1  #uncomment to debug script

    if [ $# -lt 2 ] || [ $# -gt 3 ] ; then
        echo "add2env VARNAME folder [front]"
        return
    fi

    ndir=${2%/}                    # remove trailing space
    if [ -d "$ndir" ] ; then       # if the second arg is an existing folder
        if [ -z "${!1}" ] ; then   # if the first arg is an empty env variable
            #echo $1 is empty
            eval export $1="$ndir"
        else
            #echo $1 is not empty
            if ! eval echo \$$1 | grep -E -q "(^|:)$ndir($|:)" ; then   # if ndir is not already present in the variable
                if [ "$3" == "front" ] ; then
                    eval export $1="$ndir:\$$1"  # add folder to the front
                else
                    eval export $1="\$$1:$ndir"  # add folder to the end
                fi
            else
                if [ -n "$verb" ] ; then
                    echo "! $1 already contains $ndir"
                fi
            fi
        fi
    else
        :  # null command
        #echo $2 does not exist!
    fi 

}

# arg1 is the extension of the files to be loaded
# arg2 is the name of the master node if needed (clusters)

function fct_loop()
{
    umask 007

    verb=
    #verb=1  #uncomment to debug script

    if [ -n "$verb" ] ; then
        echo "Executing \"loop.sh $*\" with shell=$0"
        #echo "script=$0"
        #echo "nargs=$#"
        #echo "arguments=$*"
    fi
    #for var in "$@"; do
    #	echo "var=$var"
    #done
    
    thisdir="$HOME/bin/cfg"

    if [ "x$2" != "x" ] && [ -d "$thisdir/$2" ]; then
        myhost=$2
    else
        myhost=`hostname -s`
    fi
    #echo "myhost=" $myhost
    
    # default cfg
    
    for i in $thisdir/default/*.$1 ; do
        #echo "testing script $i ..."
        if [ -r $i ]; then
            # do not read file if a file with the same name exists in the "myhost" folder
            if [ ! -f $thisdir/$myhost/`basename $i` ] ; then
                if [ -n "$verb" ] ; then
                    echo "  =>loading script $i ..."
                fi
                . $i
            fi
        fi
    done
    
    # machine-dependent cfg
    
    for i in $thisdir/$myhost/*.$1 ; do
        #echo "testing script $i ..."
        if [ -r $i ]; then
            if [ -n "$verb" ] ; then
                echo "  =>loading script $i ..."
            fi
            . $i
        fi
    done
    unset i


    # remove duplicate entries in env vars ----
    # see https://unix.stackexchange.com/questions/40749/remove-duplicate-path-entries-with-awk-command

    # - initial version
    #LD_LIBRARY_PATH=$(printf %s "$LD_LIBRARY_PATH" | awk -v RS=: '{ if (!arr[$0]++) {printf("%s%s",!ln++?"":":",$0)}}')
    # - loop version
    for var in PATH LD_LIBRARY_PATH INTEL_LICENSE_FILE CPATH INFOPATH NLSPATH LIBRARY_PATH MANPATH CLASSPATH MIC_LIBRARY_PATH MIC_LD_LIBRARY_PATH 
    do
        if [ -n "$(eval echo \$$var)" ] ; then # if var is set
            cmd="$var=\$(printf %s \"\$$var\" | awk -v RS=: '{ if (!arr[\$0]++) {printf(\"%s%s\",!ln++?\"\":\":\",\$0)}}')"
            eval $(echo $cmd)
        fi
    done

}
